<?php

namespace Goldenscarab\Modulus\Debusca\App\Http\Drivers\Objects;


class ResultLine
{
    private $metas;

    private $data;

    private $queries;



    public function __construct($metas, $data, $queries)
    {
        $this->metas = $metas;
        $this->data = $data;
        $this->queries = $queries;
    }

    public function getValues()
    {
        return $this->data;
    }

    public function getFirst()
    {
        return collect($this->data)->first();
    }

    public function getSelectQuery()
    {
        $replace = [
            '{TABLE}' => collect($this->getValues())->first(),
        ];

        $query = str_replace(array_keys($replace), array_values($replace), $this->queries['select']);

        return $query;
    }

    public function getInsertQuery()
    {
        $replace = [
            '{TABLE}'          => collect($this->getValues())->first(),
            '{COLUMNS_NAMES}'  => implode(', ', $this->metas->getFieldsNamesArray()),
            '{COLUMNS_VALUES}' => implode(', ', $this->metas->getFieldsDefaultArray())
        ];
        $query = str_replace(array_keys($replace), array_values($replace), $this->queries['insert']);

        return $query;
    }

    public function getUpdateQuery()
    {
        $query = '';

        if (!is_null($this->metas->getPKName())) {

            $replace = [
                '{TABLE}'        => $this->metas->getTableName(),
                '{ALIAS}'        => $this->metas->getTableAlias(),
                '{NAMES_VALUES}' => $this->getFieldNameValuesText(),
                '{PK_NAME}'      => $this->metas->getPKName(),
                '{ID}'           => $this->getValue($this->metas->getPKName())
            ];

            $query = str_replace(array_keys($replace), array_values($replace), $this->queries['update']);
        }


        return $query;
    }

    public function getDeleteQuery()
    {
        $query = '';

        if (!is_null($this->metas->getPKName())) {

            $replace = [
                '{TABLE}'   => $this->metas->getTableName(),
                '{PK_NAME}' => $this->metas->getPKName(),
                '{ID}'      => $this->getValue($this->metas->getPKName())
            ];

            $query = str_replace(array_keys($replace), array_values($replace), $this->queries['delete']);

        }

        return $query;
    }

    public function getAlterQuery()
    {
        $replace = [
            '{TABLE}'   => collect($this->getValues())->first(),
        ];

        $query = str_replace(array_keys($replace), array_values($replace), $this->queries['alter']);

        return $query;
    }

    public function getColumnsQuery()
    {
        $replace = [
            '{TABLE}'   => collect($this->getValues())->first(),
        ];

        $query = str_replace(array_keys($replace), array_values($replace), $this->queries['columns']);

        return $query;
    }

    private function getFieldNameValuesText()
    {
        $names_values = '';

        foreach ($this->metas as $meta) {

            $names_values .= vsprintf('%s.`%s` = %s, ', [
                $this->metas->getTableAlias(),
                $meta->getField(),
                $meta->isNumeric() ? $this->getValue($meta->getField()) : "'" . $this->getValue($meta->getField()) . "'",
            ]);
        }

        return trim($names_values, ', ');
    }

    private function getValue($field_name)
    {
        if (!is_null($field_name)) {
            return data_get($this->data, $field_name);
        }
    }
}
